/*! \file
 "Text Strip" to read from file -- derived from Cstrip

 vers. 2006/1/9
*/

#include "cstrip.h"

#define FSTRIP_BUFLEN 256
// actual buffer is 3 longer to account for UTF triple + terminator

struct FileStripBuf;

//! Implements 'Cstrip' for files.
/*!	In addition to basic Cstrip methods (see cstrip.h), it has methods
	to manage buffering of files.  Buffers are filled as needed by progressive
	'get' requests, and kept as long as indicated by the 'keepAfter' setting.
	(Currently there's no provision for moving 'keepAfter' backward.)
	
	Segments of text may be sent for further processing by 'processSeg'
	through the 'segProc' hook function, or returned as caller-owned
	string copies (null-terminated) by a call to the 'text' method.
*/
class FileStrip : public Cstrip {
	//! Buffer container class (internal to filestrip.cpp)
	friend struct FileStripBuf;
	char *cptr, *bufbase, *bufend;
	bool useutf;
	FILE *f;
	long  cpos, lowbound, lowbuff, topbuff, keeplow;
	FileStripBuf *buflist, *freelist, *listend, *curbuf;
	char * stepBuf(char *&cp);
	char * fillBuf();
	void keepBuf();
	void trimBufs();
  public:
	/*! Constructor
		- with optionally opened 'file'.
		- if 'utf' is true (default), expects UTF-8; otherwise 8-bit ASCII.
	*/
	FileStrip(FILE *file=NULL, bool utf=true);
	~FileStrip();
	//! Hook function that may be set to process a buffer (or partial buffer).
	/*!	It will be called with 'seg' pointing to the start of a contiguous block,
		of length 'len' (8-bit chars, not Unicode).
		It should return true normally, false if it cannot fulfill the request.
		
		The function will be called repeatedly if necessary to satisfy a
		'processSeg' request.  If the slot is NULL, processSeg is ignored.
	*/
	bool (*segProc)(char *seg, int len);
	//! Set this flag to call processSeg on pre-lowbound buffers before discard
	bool procDiscard;
	int get();
	int look();
	long pos();
	bool pos(long p);
	long lowBound(){return lowbound;}
	bool lowBound(long p);
	/*! Specify a (new) file to be read.
		If a file is currently open, it will be closed.
	*/
	bool setFile(FILE *file);
	//! Returns the lowest position still available for processing.
	long keepAfter(){return keeplow;}
	//! Sets the lowest position still available for processing.
	/*!	Cannot be moved backward!
	*/
	bool keepAfter(long p);
	//! Return a copy of the text segment delimited by 'start', 'end'.
	/*!	Numbers are positions (Unicode) not 8-bit chars.
		('end' is one beyond last char wanted.)
		
		Both start and end must refer to currently available buffers.
	*/
	char * text(long start, long end);	//  -- returns new string!
	//! Process the text segment delimited by 'start', 'end' through 'segProc'.
	/*!	Numbers are positions (Unicode) not 8-bit chars.
		('end' is one beyond last char wanted.)
		
		Both start and end must refer to currently available buffers.
	*/
	bool processSeg(long start, long end);	// calls segProc as necessary
};
